package main

import (
	"os"
	"testing"
)

func testWasmFile(t *testing.T, fname string) *Process {
	fin, err := os.Open(fname)
	if err != nil {
		t.Fatal(err)
	}
	defer fin.Close()

	p, err := NewProcess(fin, fname)
	if err != nil {
		t.Fatal(err)
	}

	ret, err := p.Main()
	if err != nil {
		t.Fatal(err)
	}

	if ret != 0 {
		t.Fatalf("expected return code to be 0, got: %d", ret)
	}

	return p
}

func TestHelloWorld(t *testing.T) {
	fin, err := os.Open("./testdata/hello.wasm")
	if err != nil {
		t.Fatal(err)
	}
	defer fin.Close()

	p, err := NewProcess(fin, "hello")
	if err != nil {
		t.Fatal(err)
	}

	ret, err := p.Main()
	if err != nil {
		t.Fatal(err)
	}

	if ret != 0 {
		t.Fatalf("expected return code to be 0, got: %d", ret)
	}

	data := p.readMem(200) // should be "Hello"
	if string(data) != "Hello" {
		t.Fatalf("wanted \"Hello\", got: %q", string(data))
	}

	_, err = p.writeMem(200, []byte("goodbye"))
	if err != nil {
		t.Fatal(err)
	}

	data = p.readMem(200)
	if string(data) != "goodbye" {
		t.Fatalf("wanted \"goodbye\", got: %q", string(data))
	}
}

func TestWriteFile(t *testing.T) {
	testWasmFile(t, "./testdata/writefile.wasm")
}

func TestFileOps(t *testing.T) {
	p := testWasmFile(t, "./testdata/fileops.wasm")
	data := p.readMem(255)
	if string(data) != "Hello, world!\n" {
		t.Fatalf("wanted \"Hello, world\", got: %q", string(data))
	}
}
